#include "Data.h"
#include <M2Datei.h>

// Konstructor
GameData::GameData( const char *shipM2, const char *asteroidsM2 )
{
    schuss = new RCArray< Schuss >();
    ship = 0;
    asteroid = new RCArray< Asteroid >();
    aData = new Model2DData*[ 7 ]();
    sData = new Model2DData*[ 2 ]();
    M2Datei m2d( asteroidsM2 );
    m2d.leseDaten();
    for( int i = 0; i < 7; i++ )
    {
        Text name = "";
        name.append( (char)( 'a' + i ) );
        aData[ i ] = m2d.ladeModel( name );
    }
    m2d.setPfad( shipM2 );
    m2d.leseDaten();
    for( int i = 0; i < 2; i++ )
    {
        Text name = "";
        name.append( (char)( 'a' + i ) );
        sData[ i ] = m2d.ladeModel( name );
    }
    shipN = 0;
    score = 0;
    scoreCheck = score * 11197;
    breite = 0;
    hhe = 0;
    aGre = 0;
    maxTimer = 0;
    mTimer = 0;
    timer = 0;
    beendet = 1;
    rend = 0;
    gameTime = 0;
    tastenStnde = 0;
    rGen = 0;
    ref = 1;
}

// Destructor
GameData::~GameData()
{
    schuss->release();
    if( ship )
        ship->release();
    asteroid->release();
    for( int i = 0; i < 7; i++ )
        aData[ i ]->release();
    for( int i = 0; i < 2; i++ )
        sData[ i ]->release();
    delete[] aData;
    delete[] sData;
    if( rGen )
        rGen->release();
}

// private
Asteroid *GameData::createNewAsteroid()
{
    int num = (int)( rGen->rand() * 7 );
    double sw = ( rGen->rand() * 360 ) / 180.0 * PI;
    Vec2< float > speed( (float)cos( sw ), (float)sin( sw ) );
    speed *= (float)( rGen->rand() * 40 );
    Vec2< float > pos( -200.f, hhe / 2.f );
    if( speed.x < 0 )
        pos.x = (float)( breite + 200 );
    return new Asteroid( aData[ num ]->getThis(), 0, pos, speed, (float)( rGen->rand() * 100 ) / 75.f, (float)sw, aGre / 1000.f, num );
}

// nicht constant
void GameData::reset( Text *zOptionen )
{
    gameTime = 0;
    timer = 0;
    beendet = 0;
    score = 0;
    scoreCheck = score * 11197;
    tastenStnde = 0;
    if( ship )
        ship = ship->release();
    schuss->leeren();
    asteroid->leeren();
    Text *tmp = zOptionen->getTeilText( zOptionen->positionVon( '=' ) + 1, zOptionen->positionVon( ',' ) );
    breite = *tmp;
    tmp->release();
    tmp = zOptionen->getTeilText( zOptionen->positionVon( '=', 1 ) + 1, zOptionen->positionVon( ',', 1 ) );
    hhe = *tmp;
    tmp->release();
    tmp = zOptionen->getTeilText( zOptionen->positionVon( '=', 2 ) + 1, zOptionen->positionVon( ',', 2 ) );
    aGre = *tmp;
    tmp->release();
    tmp = zOptionen->getTeilText( zOptionen->positionVon( '=', 3 ) + 1, zOptionen->positionVon( ',', 3 ) );
    maxTimer = *tmp;
    tmp->release();
    mTimer = maxTimer;
    tmp = zOptionen->getTeilText( zOptionen->positionVon( '=', 4 ) + 1, zOptionen->positionVon( ',', 4 ) );
    shipN = *tmp;
    tmp->release();
    Vec2< float > shipPos( (float)( breite / 2 ), (float)( hhe / 2 ) );
    Vec2< float > shipSpeed( 0, 0 );
    float shipR = (float)-PI / 2;
    if( rGen )
        rGen = rGen->release();
    rGen = new RandomGenerator();
    if( zOptionen->positionVon( '=', 6 ) >= 0 )
    {
        tmp = zOptionen->getTeilText( zOptionen->positionVon( '=', 6 ) + 1 );
        rGen->setSeed( (__int64)*tmp );
        tmp->release();
    }
    ship = new Ship( sData[ shipN ]->getThis(), shipPos, shipSpeed, shipR );
}

bool GameData::tick( double tickVal )
{
    if( beendet )
    {
        bool ret = rend;
        rend = 0;
        return ret;
    }
    cs.lock();
    gameTime += tickVal;
    // Timer
    timer -= tickVal;
    if( timer <= 0 )
    {
        mTimer -= 25;
        if( mTimer < 500 )
            mTimer = 500;
        timer = mTimer / 1000.0;
        for( int i = 0; i < rGen->rand() * 1 + 1; i++ )
        {
            asteroid->add( createNewAsteroid() );
        }
    }
    // Update
    int aAnz = asteroid->getEintragAnzahl();
    for( int i = 0; i < aAnz; i++ )
        asteroid->z( i )->tick( tickVal, breite, hhe );
    int sAnz = schuss->getEintragAnzahl();
    for( int i = 0; i < sAnz; i++ )
        schuss->z( i )->tick( tickVal );
    ship->tick( tickVal, breite, hhe, tastenStnde );
    for( int i = aAnz - 1; i >= 0; i-- )
    {
        if( !asteroid->z( i )->amLeben() )
        {
            score += asteroid->z( i )->getScore();
            scoreCheck = score * 11197;
            asteroid->remove( i );
            aAnz--;
        }
    }
    for( int i = sAnz - 1; i >= 0; i-- )
    {
        Vec2< float > pos = schuss->z( i )->getPos();
        if( pos.x < 0 || pos.y < 0 || pos.x > breite || pos.y > hhe )
        {
            sAnz--;
            schuss->remove( i );
        }
    }
    // Collision
    for( int i = sAnz - 1; i >= 0; i-- )
    {
        Schuss *zs = schuss->z( i );
        bool b = 0;
        for( int j = 0; j < aAnz; j++ )
        {
            Polygon2D ap;
            Polygon2D bp;
            Punkt pa;
            Punkt pb;
            if( asteroid->z( j )->istGetroffen( zs, ap, bp, pa, pb, rGen ) )
            {
                Array< Polygon2D > *npaA = new Array< Polygon2D >();
                npaA->add( ap );
                Model2DData *npdA = new Model2DData();
                npdA->erstelleModell( npaA );
                float rot = asteroid->z( j )->zModel()->getDrehung();
                char id = asteroid->z( j )->getId();
                if( id >= 0 )
                    id = -id - 1;
                asteroid->add( new Asteroid( npdA, asteroid->z( j )->getTextur(), pa.rotation( rot ) + asteroid->z( j )->getPos(), asteroid->z( j )->getSpeed(), asteroid->z( j )->getRSpeed(), rot, asteroid->z( j )->zModel()->getSize(), id ) );
                Array< Polygon2D > *npaB = new Array< Polygon2D >();
                npaB->add( bp );
                Model2DData *npdB = new Model2DData();
                npdB->erstelleModell( npaB );
                asteroid->add( new Asteroid( npdB, asteroid->z( j )->getTextur(), pb.rotation( rot ) + asteroid->z( j )->getPos(), asteroid->z( j )->getSpeed(), asteroid->z( j )->getRSpeed(), rot, asteroid->z( j )->zModel()->getSize(), id ) );
                b = 1;
                asteroid->z( j )->setDead();
            }
        }
        if( b )
        {
            schuss->remove( i );
            sAnz--;
        }
    }
    for( int i = 0; i < aAnz && !beendet; i++ )
        beendet |= ship->istTod( asteroid->z( i ) );
    beendet |= ( score * 11197 != scoreCheck );
    if( score * 11197 != scoreCheck )
    {
        score = 0;
        scoreCheck = 0;
    }
    cs.unlock();
    return 1;
}

// constant
int GameData::getScore() const
{
    return score;
}

bool GameData::istBeendet() const
{
    return beendet;
}

// Reference Counting
GameData *GameData::getThis()
{
    ref++;
    return this;
}

GameData *GameData::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}